/*
 * jcl.cpp, Java class launcher 
 * Copyright (C) 2002-2003 TAKAHASHI,Toru (torutk@alles.or.jp)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
// jcl.cpp : 󥽡 ץꥱѤΥȥ ݥȤ

#include <ace/SOCK_Connector.h>
#include <ace/INET_Addr.h>
#include <ace/Log_Msg.h>
#include <string>

const int DEFAULT_PORT_NUMBER = 2345;
const char* const DEFAULT_HOST_NAME = "localhost";

#if (defined(_DEBUG) || defined(DEBUG))
const ACE_Log_Priority DEFAULT_PRIORITY = LM_DEBUG;
#else
const ACE_Log_Priority DEFAULT_PRIORITY = LM_NOTICE;
#endif /* _DEBUG || DEBUG */

// 
class CommandOption
{
public:
    int port_number;
    const char* host_name;
    char* command;
    int option_length;
    char** options;
    CommandOption(int default_port_number, const char* default_host_name) :
	port_number(default_port_number), host_name(default_host_name) {}
    void print() {
	ACE_OS::printf("port_number = %d\n", port_number);
	ACE_OS::printf("host_name = %s\n", host_name);
	ACE_OS::printf("command = %s\n", command);
	ACE_OS::printf("option_length = %d\n", option_length);
	for (int i=0; i<option_length; i++) {
	    ACE_OS::printf("option[%d] = %s\n", i, options[i]);
	}
    }
};


// ץȥ
void print_usage();
bool parse_args(int argc, char* argv[], CommandOption& option);

int main(int argc, char* argv[])
{
    ACE_DEBUG((LM_DEBUG, "main entering\n"));
    // ǥХå٥뤬ꤵƤС٥ꤹ
#ifdef NDEBUG
    {
    u_long priority_mask = ACE_LOG_MSG->priority_mask(ACE_Log_Msg::PROCESS);
    ACE_CLR_BITS(priority_mask, LM_DEBUG | LM_INFO);
    ACE_LOG_MSG->priority_mask(priority_mask, ACE_Log_Msg::PROCESS);
    }
#endif

    CommandOption command_option(DEFAULT_PORT_NUMBER, DEFAULT_HOST_NAME);
    bool parseOk = parse_args(argc, argv, command_option);
    if (parseOk != true) {
	print_usage();
        ACE_ERROR_RETURN((LM_ERROR, "%p\n", "parse_args()"), -1);
    }

    ACE_SOCK_Connector connector;
    ACE_SOCK_Stream stream;
    ACE_INET_Addr addr(command_option.port_number, command_option.host_name);

    if (connector.connect(stream, addr) == -1) {
        ACE_ERROR_RETURN((LM_ERROR, "%p\n", "connector.connect()"), 1);
    }

    // 
    std::string command = command_option.command;
    for (int i=0; i<command_option.option_length; i++) {
        command.append(" ");
        command.append(command_option.options[i]);
    }
    command.append("\n");
    const char* buffer = command.c_str();
    ACE_DEBUG((LM_DEBUG, "send command is '%s'\n", buffer));
    if (stream.send_n(buffer, ACE_OS::strlen(buffer) + 1) == -1) {
        ACE_ERROR_RETURN((LM_ERROR, "%p\n", "stream.send_n()"), 1);
    }
    
    // 
    char recvBuffer[256];
    const char* RESPONSE = "Response:";

    ACE::write_n(ACE_STDOUT, RESPONSE, ACE_OS::strlen(RESPONSE));
    for (ssize_t n; (n = stream.recv(recvBuffer, sizeof recvBuffer)) > 0; ) {
        ACE::write_n(ACE_STDOUT, recvBuffer, n);
        ACE_DEBUG((LM_DEBUG, "recvBuffer[n-1]=%x\n", recvBuffer[n-1]));
        if (recvBuffer[n-1] == '\n') {
            break;
        }
    }
    
    ACE_DEBUG((LM_DEBUG, "Now closing stream\n"));
    if (stream.close() == -1) {
        ACE_ERROR_RETURN((LM_ERROR, "%p\n", "stream.close()"), 1);
    }
    
    return 0;
}

/**
 * JCLޥɤν񼰤ɽ롣
 *
 * @param argc ޥɥ饤ο
 * @param argv ޥɥ饤ʸ
 */
void print_usage()
{
    ACE_OS::printf("Java Fast Launcher Program 'Mercury' build:%s\n",
		   __DATE__ );
    ACE_OS::printf("Copyright (C) 2002, TAKAHASHI,Toru<torutk@02.246.ne.jp>\n");
    ACE_OS::printf("\nUsage:\n");
    ACE_OS::printf("\tjcl [C|R] [options...]\n");
    ACE_OS::printf("\toptions...\n");
    ACE_OS::printf("\t\tC: [URL]\n");
    ACE_OS::printf("\t\tR: [FQCN] [method name] <method argument...>\n");
}

/**
 * JCLޥɤΥޥɥ饤󥪥ץϤ򸡺롣
 * JCLޥɥ饤󥪥ץβϤϡ
 *  1) [0-9]* C URL
 *  2) [0-9]* R ClassName(FQCN) MethodName MethodArguments...
 *
 * @param argc
 * @param argv
 * @param option Ϸ̤ΥץǼ륯饹
 * @return ǤtrueǤʤfalseֵѤ
 */
bool parse_args(int argc, char* argv[], CommandOption& option)
{
    int arg_ptr = 0;
    if (argc <= 1) {
	return false;
    }
    arg_ptr++;
    ACE_DEBUG((LM_DEBUG, "parse_args(): argv[arg_ptr][0] = %c\n", argv[arg_ptr][0]));
    if ('0' <= argv[arg_ptr][0] && argv[arg_ptr][0] <= '9') {
	int port_number = ACE_OS::atoi(argv[arg_ptr]);
	ACE_DEBUG((LM_DEBUG, "parse_args(): port_number = %d\n", port_number));
	if (port_number < 0 || 65535 < port_number) {
	    return false;
	}
	arg_ptr++;
	option.port_number = port_number;
    }
    
    ACE_DEBUG((LM_DEBUG, "parse_args(): argv[arg_ptr][0] = %c\n", argv[arg_ptr][0]));
    switch (argv[arg_ptr][0]) {
    case 'C':
	option.command = argv[arg_ptr];
	arg_ptr++;
	break;
    case 'R':
	option.command = argv[arg_ptr];
	arg_ptr++;
	break;
    default:
	return false;
    }
    option.option_length = argc - arg_ptr;
    option.options = &argv[arg_ptr];
#ifndef NDEBUG
    option.print();
#endif /* NDEBUG */
    return true;
}
