/*
 * CommandDispatcher.java, Java class launcher 
 * Copyright (C) 2002-2003 TAKAHASHI,Toru (torutk@alles.or.jp)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package jp.gr.java_conf.torutk.jlauncher;

import java.util.StringTokenizer;
import java.net.URLClassLoader;
import java.net.URL;
import java.util.List;
import java.util.ArrayList;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.lang.ClassLoader;
import java.io.File;

/**
 * NGXg󂯕tsNXB
 * ɃNX[_ێĂB
 *
 * @author Toru TAKAHASHI
 * @version $Revision: 1.2 $
 */
public class CommandDispatcher {
    private MyURLClassLoader loader;
    private static final Class[] STRING_ARRAY_TYPE = {String[].class};
        
    public CommandDispatcher() {
    }
    
    /**
     * NGXg͂̕AR}hƂɏNXɐU
     * B
     * 
     * NGXgF<br>
     * [R}hL] []... CR LF
     * R}hLF[A-Z]
     * [C] NX[_ɈŎw肵NXpX(URL\L)ǉB
     *   ) C file:/d:/home/java/java_ap/
     *   LFpX̓fBNg͖̎ɕK'/'t邱ƁB
     *         ȂƁAjart@CƌȂB
     * [R] 1ڂ̈Ŏw肵O(FQCN)̃NX[hA2ڂ
     *     Ŏw肵NX\bhsBNX\bh
     *     mainȊOł悢A͕KString[]łȂĂ͂ȂȂB
     *   ) R foo.bar.MyApp main arg1 arg2 ...
     * 
     * @return R}h̏ʂ𕶎ŕԋp
     */
    public String dispatch(String request) {
        StringTokenizer st = new StringTokenizer(request);
        if (!st.hasMoreTokens()) {
            System.err.println("request has no token:" + request);
            return "FORMAT ERROR";
        }
        
        String command = st.nextToken();
        if ("C".equals(command)) {
            return processClassLoader(st);
        } else if ("R".equals(command)) {
            return processRunClass(st);
        } else {
            System.err.println("'" + command + "' cannot process");
	    return "NO SUPPORT COMMAND ERROR";
        }
    }

    /**
     * URLClassLoaderɓo^B
     * loaderCX^Xɂ͑ӑĂȏgpĂB
     * ɓo^ĂURL2do^ɂȂȂ悤Ƀ`FbN
     * rĂB
     *
     * @param st 'C'R}ḧQێg[NiCU
     * @return 
     */
    String processClassLoader(StringTokenizer st) {
        if (!st.hasMoreTokens()) {
            System.err.println("'C' command has no args");
            return "NO ARGUMENT ERROR";
        }

        try {
            URL url = new URL(st.nextToken());
            if (!isValidUrl(url)) {
                throw new MalformedURLException("URL is not valid");
            }
            
            if (loader == null) {
                loader = new MyURLClassLoader(new URL[]{url});
            } else {
                List urls = Arrays.asList(loader.getURLs());
                if (!urls.contains(url)) {
                    loader.addURL(url);
                } else {
                    return "URL already exists";
                }
            }
        } catch (MalformedURLException e) {
            System.err.println("'C' command invalid arg");
	    return "ARGUMENT ERROR (" + e.getMessage() + ")";
        } 
	return "SUCCESS";
    }

    /**
     * Ŏw肳ꂽNX[hAŎw肳ꂽ̃NX
     * static\bhĂяoB
     * static\bh́AB String[]OłB
     *
     * @param st 'R'R}ḧQێg[NiCU
     * @return 
     */
    String processRunClass(StringTokenizer st) {
        if (!st.hasMoreTokens()) {
            System.err.println("'R' command has no class name");
            return "NO CLASSNAME ERROR";
        }
        String className = st.nextToken();
        if (!st.hasMoreTokens()) {
            System.err.println("'R' command has no method name");
            return "NO METHODNAME ERROR";
        }
        String methodName = st.nextToken();

        List argList = new ArrayList();
        while (st.hasMoreTokens()) {
            argList.add(st.nextToken());
        }
        String[] args = new String[argList.size()];
        argList.toArray(args);
        try {
            Class targetClass = getClassLoader().loadClass(className);
            Method method =
                targetClass.getMethod(methodName, STRING_ARRAY_TYPE);
            method.invoke(null, new Object[]{args});
        } catch (ClassNotFoundException e) {
            System.err.println("'R' command class not found");
	    return "CLASS NOT FOUND ERROR";
        } catch (NoSuchMethodException e) {
            System.err.println("'R' command method not found");
	    return "METHOD NOT FOUND ERROR";
        } catch (IllegalAccessException e) {
            System.err.println("'R' command method invoke error");
	    return "METHOD CANNOT ACCESS ERROR";
        } catch (InvocationTargetException e) {
            e.printStackTrace();
            System.err.println("'R' command method invoke target error");
	    return "METHOD TARGET ERROR";
        }
	return "SUCCESS";
    }

    /**
     * ݕێĂNX[_ԋpB
     * NX[_ĂȂ΁AVXeNX[_
     * ԋpB
     *
     * @return gp\ȃNX[_
     */
    private ClassLoader getClassLoader() {
        if (loader == null) {
            return getClass().getClassLoader();
        } else {
            return loader;
        }
    }

    /**
     * URLÓł邩ǂB
     *
     * - URL̃vgRfileȊȌꍇ͑ÓƂ
     * - URL̃vgRfilȅꍇ́A
     *   - '/'ŏI΃fBNgۂB
     *   - '/'ȊOł΃t@CۂB
     *  ʂǂł΁AÓƔf
     *
     * @return ÓłtrueAȂfalseԋp
     */
    private boolean isValidUrl(URL url) {
        if (!"file".equals(url.getProtocol())) {
            return true;
        }
        String path = url.getPath();
        File file = new File(path);
        if (path.endsWith("/")) {
            if (file.isDirectory()) {
                return true;
            }
            return false;
        }
        if (file.isFile()) {
            return true;
        }
        return false;
    }

}// CommandDispatcher

class MyURLClassLoader extends URLClassLoader {
    MyURLClassLoader(URL[] urls) {
        super(urls);
    }

    protected void addURL(URL url) {
        super.addURL(url);
    }
}
